#!/usr/bin/perl

use Getopt::Std;
use Text::BasicTemplate;
use Net::Netmask;
use Socket;

$bt = Text::BasicTemplate->new();

getopts("h");

if($opt_h){ help(); exit; }

#my $USER=(getpwuid($<))[0];
#$home=(getpwnam($USER))[7];
#$conf_dir="$home/webscan/etc/";

require "read_config.pl";

my $params_cache="params.cache";
my $debug=1;

if( -f $params_cache and !-z $params_cache){
    %conf=read_config($params_cache);
}






if( setup() ){

my @files = ("dhcpd/dhcpd.conf","httpd/conf/httpd.conf","iptables/firewall.sh","squid/squid.conf","squid/squid.conf.CHECKER","squid/squid.conf.VAT","netreg/cgi-bin/scan_vars.pl","netreg/etc/netreg/subnet.dat","utils/mac-detect.sh");

my ($template_file,$real_file);
foreach $f (@files){

    $real_file = $f;
    $template_file = $f . ".templ";
    print "Configuring $real_file from a template file $template_file\n" if $debug; 

    open(REALFILE,">$real_file") || die "Cant open $real_file:$!\n";
    print REALFILE $bt->parse($template_file,\%conf);
    close REALFILE;
}

}


# checking if group and username presents
if(  ! getpwnam($conf{'httpd_Group'}) ){
    print "There is no such group: $conf{'httpd_Group'}\n";
    print "creating it ...\n";
    `/usr/sbin/groupadd $conf{'httpd_Group'}`; 
}

if( ! getpwnam($conf{'httpd_User'}) )
{
    print "There is no such user: $conf{'httpd_User'}\n";
    print "creating it ...\n";
    `/usr/sbin/useradd -g $conf{'httpd_Group'} -s /sbin/nologin $conf{'httpd_User'}`;
}


# uncomment this next time !!! - added May 26/05
sudoers_add();

# TODO: add error checking
sub setup
{

    print "netreg box REAL ip [$conf{'netreg_ip'}]: ";
    chomp($netreg_ip=<STDIN>); 
    $conf{'netreg_ip'}=$netreg_ip if $netreg_ip ne '';


    $netreg_dns=gethostbyaddr(inet_aton($conf{'netreg_ip'}),AF_INET);

    if( $netreg_dns eq ''){

	print "\nThe DNS name cannot be resolved for $conf{'netreg_ip'},\nthe cache has this old name $conf{'netreg_dns'},please specify new one here : ";
	chomp($netreg_dns=<STDIN>);
	$conf{'netreg_dns'}=$netreg_dns;
    }
    elsif( $conf{'netreg_dns'} ne $netreg_dns ){

        print "\nThe resolved DNS name $netreg_dns for $conf{'netreg_ip'} is different from old one $conf{'netreg_dns'}\nstored in the cache from previous configuration, please specify the new DNS name here : ";
        chomp($netreg_dns=<STDIN>);
        $conf{'netreg_dns'}=$netreg_dns;

	
    }


    print "netreg box REAL netmask [$conf{'netmask'}]: ";
    chomp($netmask=<STDIN>);
    if( $netmask ne '' ){
	$conf{'netmask'}=$netmask;
    }
    
# using this approach just to convert the mask into a bit representation
	$b=new Net::Netmask("$conf{'netreg_ip'}:$conf{'netmask'}");
	$conf{'netmask_bits'} = $b->bits();
	undef $b;
    


    print "netreg box PRIVATE ip [$conf{'netreg_priv_ip'}]: ";
    chomp($netreg_ip=<STDIN>);
    $conf{'netreg_priv_ip'}=$netreg_ip if $netreg_ip ne '';

    print "netreg box PRIVATE netmask [$conf{'priv_netmask'}]: ";
    chomp($priv_netmask=<STDIN>);
    if( $priv_netmask ne '' ){
	$conf{'priv_netmask'}=$priv_netmask;
    }
# using this approach just to convert the mask into a bit representation
	$b=new Net::Netmask("$conf{'netreg_priv_ip'}:$conf{'priv_netmask'}");
	$conf{'priv_netmask_bits'} = $b->bits();
	undef $b;
    


    print "the REAL subnet [$conf{'real_subnet'}]: ";
    chomp($real_subnet=<STDIN>);
    $conf{'real_subnet'}=$real_subnet if $real_subnet ne '';
    
    print "the PRIVATE subnet [$conf{'priv_subnet'}]: ";
    chomp($priv_subnet=<STDIN>);
    $conf{'priv_subnet'}=$priv_subnet if $priv_subnet ne '';

    print "Gateway ip [$conf{'gateway_ip'}]: ";
    chomp ($gateway_ip=<STDIN>);
    $conf{'gateway_ip'}=$gateway_ip if $gateway_ip ne '';

    print "domain name [$conf{'domain_name'}]: ";
    chomp ($domain_name=<STDIN>);
    $conf{'domain_name'}=$domain_name if $domain_name ne '';

    print "DNS server ip [$conf{'dns_ip'}]: ";
    chomp ($dns_ip=<STDIN>);
    $conf{'dns_ip'}=$dns_ip if $dns_ip ne '';

# possible value could be 3000
    print "MAC Address blocking feature - Bandwidth Limiting packet rate [$conf{'bandwidth_limit_pktrate'}]: ";
    chomp ($bandwidth_limit_pktrate=<STDIN>);
    $conf{'bandwidth_limit_pktrate'}=$bandwidth_limit_pktrate if $bandwidth_limit_pktrate ne '';

# possible value could be 24000
    print "MAC Address blocking feature - Bandwidth Limiting packet burst [$conf{'bandwidth_limit_pktburst'}]: ";
    chomp ($bandwidth_limit_pktburst=<STDIN>);
    $conf{'bandwidth_limit_pktburst'}=$bandwidth_limit_pktburst if $bandwidth_limit_pktburst ne '';


    print "DHCP REAL pool range [$conf{'dhcp_real_pool_range'}]: ";
    chomp ($dhcp_real_pool_range=<STDIN>);

    if( $dhcp_real_pool_range ne '' ){

    $conf{'dhcp_real_pool_range'}=$dhcp_real_pool_range;
}
    
    # we need to specify a number of registered leases for /etc/netreg/subnet.dat 
    ($startip,$endip)=$conf{'dhcp_real_pool_range'} =~ /\s*(.+)\s+(.+)\s*/;

    # get list of net blocks defined by the ip range
    @blks=range2cidrlist($startip,$endip);
    
    $num_leases_for_reg_clients=0;
    for (@blks){
	
	$num_leases_for_reg_clients += $_->size();
    }
    
    $conf{'num_leases_for_reg_clients'}=$num_leases_for_reg_clients;
    
    


    print "DHCP PRIVATE pool range [$conf{'dhcp_priv_pool_range'}]: ";
    chomp ($dhcp_priv_pool_range=<STDIN>);
    $conf{'dhcp_priv_pool_range'}=$dhcp_priv_pool_range if $dhcp_priv_pool_range ne '';


    print "DHCP REAL address pool lease-time (seconds) [$conf{'dhcp_real_pool_lease_time'}]: ";
    chomp ($dhcp_real_pool_lease_time=<STDIN>);
    $conf{'dhcp_real_pool_lease_time'}=$dhcp_real_pool_lease_time if $dhcp_real_pool_lease_time ne '';

    print "DHCP PRIVATE address pool lease-time (seconds) [$conf{'dhcp_priv_pool_lease_time'}]: ";
    chomp ($dhcp_priv_pool_lease_time=<STDIN>);
    $conf{'dhcp_priv_pool_lease_time'}=$dhcp_priv_pool_lease_time if $dhcp_priv_pool_lease_time ne '';




    print "httpd USER [$conf{'httpd_User'}]: ";
    chomp ($httpd_User=<STDIN>);
    $conf{'httpd_User'}=$httpd_User if $httpd_User ne '';

    print "httpd GROUP [$conf{'httpd_Group'}]: ";
    chomp ($httpd_Group=<STDIN>);
    $conf{'httpd_Group'}=$httpd_Group if $httpd_Group ne '';

    print "httpd ServerAdmin email [$conf{'httpd_ServerAdmin_email'}]: ";
    chomp ($httpd_ServerAdmin_email=<STDIN>);
    $conf{'httpd_ServerAdmin_email'}=$httpd_ServerAdmin_email if $httpd_ServerAdmin_email ne '';

    print "httpd ServerName [$conf{'httpd_ServerName'}]: ";
    chomp ($httpd_ServerName=<STDIN>);
    $conf{'httpd_ServerName'}=$httpd_ServerName if $httpd_ServerName ne '';

    print "httpd Access Control List [$conf{'httpd_acl_allow_from'}]: ";
    chomp ($httpd_acl_allow_from=<STDIN>);
    $conf{'httpd_acl_allow_from'}=$httpd_acl_allow_from if $httpd_acl_allow_from ne '';

#    print "VAT scanner: ip (has to be in the private subnet, probably the same as netreg box private ip) [$conf{'vat_scanner_ip'}]: ";
#    chomp ($vat_scanner_ip=<STDIN>);
    
    $vat_scanner_ip=$conf{'netreg_priv_ip'};
    $conf{'vat_scanner_ip'}=$vat_scanner_ip if $vat_scanner_ip ne '';

#    print "VAT scanner: port [$conf{'vat_scanner_port'}]: ";
#    chomp ($vat_scanner_port=<STDIN>);
    $vat_scanner_port=9090;
    $conf{'vat_scanner_port'}=$vat_scanner_port if $vat_scanner_port ne '';

#    print "VAT scanner: scan type (recommended: WIN1) [$conf{'vat_scanner_scantype'}]: ";
#    chomp ($vat_scanner_scantype=<STDIN>);
    $vat_scanner_scantype="WIN1";
    $conf{'vat_scanner_scantype'}=$vat_scanner_scantype if $vat_scanner_scantype ne '';



    

    print "\nDo you want to save these settings in the $params_cache file [y|N]: ";
    my $answer=<>;

    if($answer =~ /y/i){
        print "Saving new settings in $params_cache file ...\n";
	open(CACHE,">$params_cache") || die "Can't save settings in $params_cache:$!\n";
	for (keys %conf){
	    print CACHE "$_=$conf{$_}\n";
	}
	
	close CACHE;
 
	print "\nDo you want to set new network parameters\n/etc/sysconfig/network-scripts/ifcfg-eth* will be overwritten [y|N]: ";
	$answer=<>;
	if($answer =~ /y/i){
	    net_config();
	}

	return 1;
    }

    return 0;
}


# setting up the network
sub net_config
{
   my  $res=`/sbin/ifconfig`;
   my ($dev,$mac)=($res =~ /([^\s]+) .*Link.*HWaddr (.+)/);

my $if_file="net/ifcfg-$dev";
   open(IFCFG,">$if_file") || die "Cant open $if_file:$!\n";

    print IFCFG <<EOM;
# Generated by NetReg+ESP config.pl script
DEVICE=$dev
BOOTPROTO=none
HWADDR=$mac
IPADDR=$conf{'netreg_ip'}
NETMASK=$conf{'netmask'}
ONBOOT=yes
TYPE=Ethernet
USERCTL=no
PEERDNS=no
GATEWAY=$conf{'gateway_ip'}
IPV6INIT=no
EOM

close IFCFG;

my $if_file1="net/ifcfg-$dev:1";
open(IFCFG1,">$if_file1") || die "Cant open $if_file1:$!\n";

print IFCFG1 <<EOM1;
# Genarated by NetReg+ESP config.pl script
DEVICE=$dev:1
BOOTPROTO=none
HWADDR=$mac
IPADDR=$conf{'netreg_priv_ip'}
NETMASK=$conf{'priv_netmask'}
ONBOOT=yes
TYPE=Ethernet
USERCTL=no
PEERDNS=no
IPV6INIT=no
EOM1

close IFCFG1;

}


sub sudoers_add
{

my $hostname=`hostname`;
chomp $hostname;
my $entry="$conf{'httpd_User'} $hostname = NOPASSWD: /usr/local/bin/netreg_block_mac.sh";


my $sudoers=0;
if( -f "/etc/sudoers" ){

    open(SUDOERS,"/etc/sudoers") || die "Can't open /etc/sudoers:$!\n";

    while(<SUDOERS>){

        chomp;

        if($_ =~ /$entry/){
            $sudoers=1;
        }

    }

    close SUDOERS;
}

if( !$sudoers ){

    `cp /etc/sudoers /etc/sudoers.bak`;
    `echo $entry >> /etc/sudoers.bak `;
    $status=system("visudo -q -c -f /etc/sudoers.bak");

    if($status == 0){

        `mv -f /etc/sudoers.bak /etc/sudoers`;
    }

}

}



sub help
{
    print <<EOH;
Usage: $0 [-h]
EOH
}
